#!/usr/bin/perl -w
use strict;
use Getopt::Long;
use File::Basename;


###############################################################
##################### DOCUMENTATION ###########################

## Author: Yun Li
## Date: 2009-08-12
## Goal: Extract columns for desired SNPs from imputed .(ml)dose and .(ml)geno files
my $location = $0;
my $title = "ExtractImputed.pl Version 0.0.4";

print "$title\n";
print "\t\@: $location\n";
print "\tby: Yun Li(ylwtx\@umich.edu)\n\n";

## Input Parameters
##

## Input Files
## (1) -i        .mldose or .mlgeno
## (2) -s        imp.snps
## (3) -r        relevant.snps

## Output Files
## (1) .mldose or .mlgeno
## (2) .snps (intersection of relevant.snps and imp.snps)

################# END OF DOCUMENTATION ########################
###############################################################
my $start = time();
my $MIN_NUM_OPTS = 4;

if($#ARGV<($MIN_NUM_OPTS*2 - 1))
{
	&usage();
	die "Failed to parse parameters\n\n";
}


my %opts = ();

	# Default Options

Getopt::Long::GetOptions(\%opts,qw(
	i=s
	s=s
  r=s
  o=s
  info=s
)) || die "Failed to parse options\n\n";

if ( (!defined $opts{i}) || (!defined $opts{s}) || (!defined $opts{r}) || (!defined $opts{o}) ) 
{
	&usage();
	die "Failed to parse parameters\n\n";
}

&printPar();


my %snps = (); # key  : SNP name;
               # value: SNP index (starting at 2);
my @outputsnps = (); # array of output SNPs: in $opts{r} order;
my @outputindices = (); # array of indices for output SNPs;
my $postfix = "mldose"; 
&main();


# begin sub

sub usage
{
	print "\n";
	print "Usage:\n\t";
	print "-i\t Input (Required, e.g., XX.mldose or XX.geno) \n\t";
	print "-s\t List of imputed SNPs (Required, e.g., imp.snps) \n\t";
	print "-r\t List of (relevant) SNPs desired (Required) \n\t";
	print "-o\t Output Prefix (Required) \n\t";
	print "-info\t Input info file (Optional, if provided, output info file will be generated) \n\t";
	print "\n";
}


sub printPar
{
	print "\n";
	print "Parameters in Effect:\n";
	print "\t Input \n\t\t-i '$opts{i}'\n";
	print "\t List of imputed SNPs \n\t\t-s '$opts{s}'\n";
	print "\t List of (relevant) SNPs desired \n\t\t-r '$opts{r}'\n";
	print "\t Output Prefix \n\t\t-o '$opts{o}'\n";
	if (defined $opts{info}) {print "\t Input info file \n\t\t-info '$opts{info}'\n";}
	print "\n";
}


sub rm
{
	if(-e $_[0])
	{
		print "WARNING: $_[0] existed and removed!!\n\n";
		system "/bin/rm -rf $_[0]";
	}
}



sub bakrm
{
	if(-e $_[0])
	{
		my $tt = $_[0].".old";
		print "WARNING: $_[0] existed and moved to $tt !!\n\n";
		system "mv $_[0] $tt";
	}
}


sub swap
{
	my $tmp = $_[0];
	$_[0] = $_[1];
	$_[1] = $tmp;
}


sub main
{
  &loadSNPs();
  &loadRelevant();
  &processImputedResults();

  if (defined $opts{info}) {&processInfo();}

  my $end = time();
  my $diff = $end - $start;
  print "\nAnalysis took $diff seconds\n\n";

}


sub loadSNPs
{
  open(IN, $opts{s}) || die "cannot open file $opts{s}\n\n";
  my $linenum = 2;
  while(my $line = <IN>)
  {
    chomp($line);
    if (exists $snps{$line}) {die "SNP $line is duplidated\n\n";}
    
    $snps{$line} = $linenum;
    $linenum++;
  }
  close(IN);
  print "Read ". keys(%snps) ." SNPs from $opts{s}\n";
}

sub loadRelevant
{
  my $of = $opts{o}.".extracted.snps";
  &rm($of);
  open(OUT, ">>$of") || die "cannot create file $of\n\n";
  
  open(IN, $opts{r}) || die "cannot open file $opts{r}\n\n";
  my $linenum = 0;
  while(my $line = <IN>)
  {
    $linenum ++;
    chomp($line);
    if (!exists $snps{$line}) 
    {
      print "WARNING: $line NOT imputed\n";
      next;
    }
    push @outputsnps, $line;
    print OUT "$line\n";
    push @outputindices, $snps{$line};
    
  }
  close(IN);
  close(OUT);
  
  printf("%d of %d SNPs will be in the output\n", scalar(@outputsnps), $linenum);
  if (scalar(@outputsnps) == 0)
  {
    print "Nothing to output\n\n";
    &rm($of);
    exit;
  }
}



sub processImputedResults
{
  # Open the files for reading  (files may be compressed)
  if ($opts{i} =~ /(.+)\.gz$/)
  {
    $opts{i} = $1;
  } # Remove gz suffix if provided                
  if (-r "$opts{i}.gz")
  {
    $opts{i} = "$opts{i}.gz";
    open(IN, "gunzip -c $opts{i} |") || die "Unable to open compressed file '$opts{i}': $!\n";
  }
  else
  {
    open(IN, $opts{i}) || die "Unable to open file '$opts{i}': $!\n";
  }
  
  if($opts{i} =~ m/\.mldose/) {$postfix = ".mldose";}
  elsif($opts{i} =~ m/\.dose/) {$postfix = ".dose";}
  elsif($opts{i} =~ m/\.mlgeno/) {$postfix = ".mlgeno";}
  elsif($opts{i} =~ m/\.geno/) {$postfix = ".geno";}
  else 
  {
    print "ERROR: Wrong file type: only .(ml)geno and .(ml)dose are allowed\n\n";
    exit;
  }
  
  my $of = $opts{o}.".extracted".$postfix;
  &rm($of);
  open(OUT, ">>$of") || die "cannot create file $of\n\n";
  
  my $linenum = 0;
  while(my $line = <IN>)
  {
    $linenum ++;
    chomp($line);
    my @linearr = (split(/\s+/, $line));
    if (scalar(@linearr) == 0) {next;}
    if (scalar(@linearr) != keys(%snps) + 2) 
    {
      printf("ERROR: line $linenum has %d fields while expecting %d fields\n", scalar(@linearr), (keys(%snps) + 2));
      exit;
    }
    print OUT "$linearr[0] $linearr[1] ";
    for (my $i = 0; $i < $#outputsnps; $i++)
    {
      print OUT "$linearr[$outputindices[$i]] ";
    }
    # last SNP
    print OUT "$linearr[$outputindices[$#outputsnps]]\n";
  }
  close(IN);
  close(OUT);
}

sub processInfo
{
  ## load list of desired SNPs
  my %desiredSNPs = ();
  open(IN, $opts{r}) || die "cannot open file $opts{r}\n\n";
  my $linenum = 0;
  while(my $line = <IN>)
  {
    $linenum ++;   
    chomp($line);
    if (exists $desiredSNPs{$line})
    {
      print "WARNING $line is duplicated in the wish list of SNPs\n";
    }
    else
    {
      $desiredSNPs{$line} = $linenum;
    }
  }
  close(IN);
  my %infohash = ();

  my $oinfof = $opts{o}.".extracted.info";
  open(IN, $opts{info}) || die "cannot open file $opts{info}\n\n";
  &rm($oinfof);
  open(OUT, ">>$oinfof") || die "cannot create file $oinfof\n\n";
  my $firstline = <IN>;
  chomp($firstline);
  ## print out header line
  print OUT "$firstline\n";
  $linenum = 1;
  while(my $line = <IN>)
  {
    $linenum ++;
    chomp($line);
    my @linearr = (split(/\s+/, $line));
    if (scalar(@linearr) == 0) {next;}
    #if (scalar(@linearr) != 7)
    #{
      #print "ERROR $line $linenum in info file is invalid\n\n";
      #exit;
    #}
    if (exists $desiredSNPs{$linearr[0]}) { $infohash{$desiredSNPs{$linearr[0]}} = "$line";}
  }
  close(IN);
  
  foreach my $c (sort {$a <=> $b} keys %infohash)
  {
    print OUT "$infohash{$c}\n";
  }
  close(OUT);
}


